;*** SA with LCD  PIC 16F877 [A] - S53MV - 02.01.2007 ***

;*** modify by alexp 2009-08-16 ***
;    for MCU PIC16F877 [A]
;    Generate signals:
;       DAC output (RD0-7)
;***

;*** Ports - clock 24MHz ***
;analog  inp RA0 - SA VIDEO inp
;digital inp RA1 - switch MINMAX
;analog  -Vref RA2 - area A/D
;analog  +Vref RA3 - area A/D
;digital inp RA4 - 80dB/40dB area
;digital inp RA5 - off/on gird
;digital inp RB0 - TRIGGER inp
;digital inp RB1 - LCD D7 busy
;digital out RB2 - LCD /CS1
;digital out RB3 - LCD /CS2
;digital out RB4 - LCD /RSTB
;digital out RB5 - LCD R/W
;digital out RB6 - LCD D/I
;digital out RB7 - LCD E
;digital out RC0-7 - LCD D0-7

;*** RAM definition ***
;20     ZANKA counter column
;21,22  ZANKA, STOLP value column
;23     STOLP counter line
;24,25  STOLP calculate picture
;26,27  ZANKA gird

;30-6F,B0-EF,130-16F,1B0-1EF    INTR memory 4x64 bytes

;70,71  INTR save W and STATUS
;72     INTR counter samples
;73     INTR counter column
;74     INTR old value A/D
;75,76  INTR min in max
;77     INTR save FSR

PORTD   EQU     08
PORTE   EQU     09

;*** Begin programm ***
        LIST    P=16F877
        ORG     0
ZACETEK
        NOP
        CALL    INIPIC  ;init PIC
        CALL    INILCD  ;turn on LCD two controllers
        GOTO    ZANKA

;*** Interrupt handler: A/D and TRIGGER *** (70,71,72,73,74,75,76,77)
        ORG     4
INTR
        MOVWF   70      ;save W
        SWAPF   3,0     ;save STATUS
        MOVWF   71
        CLRF    3       ;clear from memory IRP, RP1, RP0
        MOVLW   8C      ;set TMR0 to next process
        MOVWF   1
        BCF     0B,2    ;reset T0IF
        BTFSS   0B,1    ;INT TRIGGER?
        GOTO    INTR1
        MOVLW   0FE     ;delay average
        MOVWF   73
        MOVLW   14      ;delay fine
        MOVWF   72
        CLRF    1       ;reset TMR0
        BCF     0B,1    ;reset INTF
        GOTO    INTR3
INTR1
        BTFSC   1F,2    ;wait for end of previous A/D process
        GOTO    INTR1

        movf    73,0    ;put next value to DAC
        addlw   2
        movwf   PORTD

        RRF     1E,0    ;rezult A/D process div/2 to (74)
        BSF     1F,2    ;begin new A/D process
        MOVWF   74
        BCF     74,7
        MOVF    75,0    ;store minimum to (75)
        SUBWF   74,0
        BTFSS   3,0
        ADDWF   75,1
        MOVF    76,0    ;store maksimum to (76)
        SUBWF   74,0
        BTFSC   3,0
        ADDWF   76,1
        DECFSZ  72,1    ;counter samples in column
        GOTO    INTR3
        MOVLW   8
        MOVWF   72
        INCF    73,1    ;check overflow counter column
        BTFSC   73,7
        GOTO    INTR2
        MOVF    4,0     ;save FSR
        MOVWF   77
        RLF     73,0    ;calculate addr to (4) and IRP
        ANDLW   3E
        ADDLW   30
        MOVWF   4
        BTFSC   73,5
        BSF     4,7
        BTFSC   73,6    ;set IRP
        BSF     3,7
        BTFSS   5,1     ;MINMAX memory ON?
        GOTO    INTR4
        MOVF    75,0    ;store min and max to memory (4)
        MOVWF   0
        INCF    4,1
        MOVF    76,0
        MOVWF   0
        GOTO    INTR5
INTR4
        MOVF    0,0     ;store min to memory
        SUBWF   75,0
        BTFSS   3,0
        ADDWF   0,1
        INCF    4,1
        MOVF    0,0     ;store max to memory+1
        SUBWF   76,0
        BTFSC   3,0
        ADDWF   0,1
INTR5
        MOVF    77,0    ;restore FSR
        MOVWF   4
INTR2
        MOVF    74,0    ;new min and max value of previous process
        MOVWF   75
        MOVWF   76
INTR3
        SWAPF   71,0    ;restore STATUS
        MOVWF   3
        SWAPF   70,1    ;restore W
        SWAPF   70,0
        RETFIE


;*** init PIC 16F876/16F876A ***
INIPIC
        CLRF    5       ;first status of gate
        CLRF    6
        CLRF    7
        BSF     3,5     ;set PAGE1
        MOVLW   8F      ;OPTION: 1=/RBPU, 0=INTEDG, 0=T0CS, 0=T0SE, 1=PSA, 111=PS2-0
        MOVWF   1
        MOVLW   3F      ;inp RA0-5
        MOVWF   5
        MOVLW   3       ;inp RB0-1, out RB2-7
        MOVWF   6
        MOVLW   0       ;out RC0-7
        MOVWF   7

        movlw   0       ;set out RD0-7 (DAC)
        movwf   08      ;TRISD

;        movlw   04      ;set out RE0 (frame strobe)
;        movwf   09      ;TRISE

        MOVLW   0F      ;ADCON1: 0=ADFM, 000, 1111=PCFG3-0
        MOVWF   1F
        BCF     3,5     ;set PAGE0

        movlw   0       ;init DAC
        movwf   PORTD
;        movlw   1       ;init frame strobe
;        movwf   PORTE

        MOVLW   81      ;ADCON0: 10=ADCS1-0, 000=CHS2-0, 0=GO/DONE, 0, 1=ADON
        MOVWF   1F
        CLRF    1       ;reset TMR0
        BSF     0B,4    ;turn on INTE
        BSF     0B,5    ;turn on T0IE
        BSF     0B,7    ;turn on GIE
        RETURN

;*** Main loop - write to LCD *** (20,21,22,27,28,29,2A)
ZANKA
        INCF    20,1    ;counter of columns
        BTFSS   20,7
        GOTO    ZANKA1
        CALL    INILCD  ;turn LCD two controllers
        MOVLW   0F4     ;first value gird
        MOVWF   26
        CLRF    20
ZANKA1
        RLF     20,0    ;calculate addr to (4) in IRP
        ANDLW   3E
        ADDLW   30
        MOVWF   4
        BTFSC   20,5
        BSF     4,7
        BCF     3,7     ;clear or set IRP
        BTFSC   20,6
        BSF     3,7
        BTFSS   5,4     ;select direction 80dB/40dB RA4?
        GOTO    ZANKA2
        RRF     0,0     ;store min/2 to (21) in area 80dB
        MOVWF   21
        BCF     21,7
        INCF    4,1
        RRF     0,0     ;store max/2+1 to (22)
        MOVWF   22
        BCF     22,7
        INCF    22,1
        GOTO    ZANKA3
ZANKA2
        MOVLW   20      ;store min-32 to (21) in area 40dB
        SUBWF   0,0
        MOVWF   21
        INCF    4,1
        MOVLW   1F      ;store max-31 to (22)
        SUBWF   0,0
        MOVWF   22
        BTFSC   21,7    ;check (21)
        CLRF    21
        BTFSC   22,7    ;check (22)
        CLRF    22
ZANKA3
        CLRF    27      ;calculate gird on screen to (27)
        MOVLW   7
        ANDWF   20,0
        BTFSC   3,2
        INCF    26,1
        MOVLW   14
        ADDWF   26,1
        BTFSC   5,5     ;off gird RA5?
        BCF     3,0
        BTFSC   3,0
        BSF     27,0    ;more 8 points to (27)
        CALL    STOLP   ;write column to LCD
        GOTO    ZANKA

;*** Write column (21,22) with gird (27) to (20) *** (21,22,23,24,25)
STOLP
        MOVLW   0C      ;select half of screen
        IORWF   6,1
        MOVLW   0FB
        BTFSC   20,6
        MOVLW   0F7
        ANDWF   6,1
        MOVLW   7       ;counter divide (23)
        MOVWF   23
STOLP1
        MOVLW   0FF     ;calculate picture (21) in (24)
        MOVWF   24
        MOVLW   8
        SUBWF   21,1
        BTFSC   3,0
        GOTO    STOLP2
        CLRF    24
        ADDWF   21,1
        BTFSC   3,2
        GOTO    STOLP2
        MOVLW   0F0
        BTFSC   21,2
        MOVWF   24
        BSF     3,0
        BTFSC   21,1
        RRF     24,1
        BSF     3,0
        BTFSC   21,1
        RRF     24,1
        BSF     3,0
        BTFSC   21,0
        RRF     24,1
        CLRF    21
STOLP2
        MOVF    20,0    ;LCD command Y address = column
        ANDLW   3F
        IORLW   40
        CALL    UKAZ
        MOVLW   0FF     ;calculate picture (22) in (25)
        MOVWF   25
        MOVLW   8
        SUBWF   22,1
        BTFSC   3,0
        GOTO    STOLP3
        CLRF    25
        ADDWF   22,1
        BTFSC   3,2
        GOTO    STOLP3
        MOVLW   0F0
        BTFSC   22,2
        MOVWF   25
        BSF     3,0
        BTFSC   22,1
        RRF     25,1
        BSF     3,0
        BTFSC   22,1
        RRF     25,1
        BSF     3,0
        BTFSC   22,0
        RRF     25,1
        CLRF    22
STOLP3
        MOVF    23,0    ;LCD command X address = line
        IORLW   0B8
        CALL    UKAZ
        MOVF    27,0    ;gird (27) on screen
        XORWF   24,0    ;picture (24) on screen
        XORWF   25,0    ;picture (25) on screen
        BTFSS   20,7    ;delete columns >127
        CALL    BAJT    ;LCD data = 8-bit segment
        DECF    23,1
        BTFSS   23,7
        GOTO    STOLP1
        RETURN

;*** turn on and init LCD two controllers ***
INILCD
        BSF     6,4     ;set /RSTB
        BSF     6,2     ;1=/CS1
        BCF     6,3     ;0=/CS2
        MOVLW   3F      ;display ON
        CALL    UKAZ
        MOVLW   0C0     ;display start line 0
        CALL    UKAZ
        BCF     6,2     ;0=/CS1
        BSF     6,3     ;1=/CS2
        MOVLW   3F      ;display ON
        CALL    UKAZ
        MOVLW   0C0     ;display start line 0
        CALL    UKAZ
        RETURN

;*** check BUSY and send data from W ***
BAJT
        BCF     6,7     ;0=enable
        BSF     6,5     ;1=read
        BCF     6,6     ;0=instruction
        BSF     6,7     ;1=enable
        CLRF    7       ;null on D0-7
        NOP
        BTFSC   6,1     ;check D7=BUSY
        GOTO    BAJT
        BCF     6,7     ;0=enable
        MOVWF   7       ;W to D0-7
        BCF     6,5     ;0=write
        BSF     6,6     ;1=data
        BSF     6,7     ;1=enable
        NOP
        NOP
        BCF     6,7     ;0=enable
        RETURN

;*** check BUSY and send command from W ***
UKAZ
        BCF     6,7     ;0=enable
        BSF     6,5     ;1=read
        BCF     6,6     ;0=instruction
        BSF     6,7     ;1=enable
        CLRF    7       ;null to D0-7
        NOP
        BTFSC   6,1     ;check D7=BUSY
        GOTO    UKAZ
        BCF     6,7     ;0=enable
        MOVWF   7       ;W to D0-7
        BCF     6,5     ;0=write
        BSF     6,7     ;1=enable
        NOP
        NOP
        BCF     6,7     ;0=enable
        RETURN

;*** Configuration of MCU PIC 16F876/16F876A ***
;        ORG     2007
;        DW      3972
;3: 11=/CP=off
;9: 1=/DEBUG=off, 00=WRT10=off, 1=/CPD=off
;7: 0=LVP, 1=BODEN=on, 11=/CP=off
;2: 0=/PWRTE=on, 0=WDTE=off, 10=HSoscillator

        END

;*** eof ***
